from ralium.element import HTMLElement
from ralium.webpage import WebHook
from ralium.config import WindowConfig

import webview

class Window:
    def __init__(self, webhooks: list[WebHook], config: WindowConfig | None = None) -> None: ...
    
    def getElementById(self, id: str) -> HTMLElement | None: ...
    def load_handler(self, window: webview.Window) -> None: ...
    def shutdown(self) -> None: ...
    def display(self, url: str) -> None: ...
    def render(self, html: str) -> None: ...
    def start(self) -> None: ...
    def show(self) -> None: ...
    def hide(self) -> None: ...